// -----------------------------------------------------------------------
// MatchPoint - DKFZ translational registration framework
//
// Copyright (c) German Cancer Research Center (DKFZ),
// Software development for Integrated Diagnostics and Therapy (SIDT).
// ALL RIGHTS RESERVED.
// See mapCopyright.txt or
// http://www.dkfz.de/en/sidt/projects/MatchPoint/copyright.html
//
// This software is distributed WITHOUT ANY WARRANTY; without even
// the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
// PURPOSE.  See the above copyright notices for more information.
//
//------------------------------------------------------------------------
/*!
// @file
// @version $Revision: 797 $ (last changed revision)
// @date    $Date: 2014-10-10 11:42:05 +0200 (Fr, 10 Okt 2014) $ (last change date)
// @author  $Author: floca $ (last changed by)
// Subversion HeadURL: $HeadURL: https://svn.inet.dkfz-heidelberg.de/sbr/Sources/SBR-Projects/MatchPoint/trunk/Code/Core/include/mapRegistrationManipulator.h $
*/

#ifndef __REGISTRATION_MANIPULATOR_H
#define __REGISTRATION_MANIPULATOR_H

#include "mapRegistration.h"
#include "mapRegistrationBase.h"

namespace map
{
	namespace core
	{


		/*! @class RegistrationBaseManipulator
		    @brief Grants public write access to RegistrationBase.
		 */
		class RegistrationBaseManipulator
		{
		public:
			typedef RegistrationBase::TagMapType TagMapType;

			/*! @brief sets the tags of a registration
			@eguarantee strong
			*/
			void setTagValues(const TagMapType& tags)
			{
				_pBase->setTagValues(tags);
			};

			/*! @brief allows non const access to the registration tags.
			@eguarantee no fail
			*/
			TagMapType& getTagValues()
			{
				return _pBase->getTagValues();
			};

			RegistrationBaseManipulator(RegistrationBase* pBase): _pBase(pBase)
			{
				assert(pBase);
			};

		protected:
			RegistrationBase* _pBase;

		private:
			//No copy constructor allowed
			RegistrationBaseManipulator(const RegistrationBaseManipulator& source);
			void operator=(const RegistrationBaseManipulator&);  //purposely not implemented
		};

		/*! @class RegistrationManipulator
		    @brief Grants public write access to Registration classes
		 */
		template<typename TRegistration>
		class RegistrationManipulator
		{
		public:
			typedef TRegistration RegistrationType;

			/*! sets the DirectMapping kernel
			  @param pKernel the kernel for direct mapping
			  @eguarantee no fail
			 */
			virtual void setDirectMapping(typename RegistrationType::DirectMappingType* pKernel);

			/*! sets the InverseMapping kernel
			  @param pKernel the kernel for inverse mapping
			  @eguarantee no fail
			 */
			virtual void setInverseMapping(typename RegistrationType::InverseMappingType* pKernel);

			typedef typename RegistrationType::TagMapType TagMapType;

			/*! @Sets the tags of a registration by replacing the existing tag map with the passed one.
			 * @remark If tags do not contain the tag "RegistrationUID" it will be added automatically.
			 * If it is present it stays untouched. If you define your own UID ensure that it is unique.
			@eguarantee strong
			*/
			virtual void setTagValues(const TagMapType& tags);

			/*! @brief allows non const access to the registration tags.
			@eguarantee no fail
			*/
			virtual TagMapType& getTagValues();

			RegistrationManipulator(RegistrationType* pI);

		protected:
			RegistrationType* _pInterface;

		private:
			//No copy constructor allowed
			RegistrationManipulator(const RegistrationManipulator& source);
			void operator=(const RegistrationManipulator&);  //purposely not implemented
		};

	}
}

#ifndef MatchPoint_MANUAL_TPP
# include "mapRegistrationManipulator.tpp"
#endif

#endif
